#include "texture.h"
#include "texturing.h"
#include "texunit.h"

using namespace std;

namespace Msp {
namespace GL {

Texturing::~Texturing()
{
	if(current()==this)
		unbind();
}

void Texturing::attach(unsigned attch, const Texture &tex)
{
	set_attachment(attch, &tex);
}

void Texturing::detach(unsigned attch)
{
	set_attachment(attch, 0);
}

void Texturing::set_attachment(unsigned attch, const Texture *tex)
{
	if(attch>=TexUnit::get_n_units())
		throw out_of_range("Texturing::set_attachment");

	if(attachments.size()<=attch)
		attachments.resize(attch+1);

	attachments[attch] = tex;

	if(current()==this)
		bind_attachment(attch);
}

const Texture *Texturing::get_attached_texture(unsigned i) const
{
	return i<attachments.size() ? attachments[i] : 0;
}

void Texturing::bind_attachment(unsigned i) const
{
	if(const Texture *tex = attachments[i])
		tex->bind_to(i);
	else
		Texture::unbind_from(i);
}

void Texturing::unbind_attachment(unsigned i)
{
	Texture::unbind_from(i);
}

void Texturing::bind() const
{
	const Texturing *old = current();
	if(set_current(this))
	{
		for(unsigned i=0; i<attachments.size(); ++i)
			bind_attachment(i);
		if(old)
		{
			for(unsigned i=attachments.size(); i<old->attachments.size(); ++i)
				unbind_attachment(i);
		}
	}
}

void Texturing::unbind()
{
	const Texturing *old = current();
	if(set_current(0))
	{
		for(unsigned i=old->attachments.size(); i--;)
			unbind_attachment(i);
	}
}

} // namespace GL
} // namespace Msp;
